#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>
#include <strings.h>
#include <glade/glade.h>
#include <gmpc/plugin.h>
#include "open-location.h"


#define MAX_PLAYLIST_SIZE 10000

#include <libgnomevfs/gnome-vfs.h>


GladeXML *ol_xml = NULL;
GnomeVFSAsyncHandle *handle = NULL;
GnomeVFSAsyncHandle *handle1 = NULL;
/* this is so I won't return while busy */

extern gmpcPlugin plugin;

void ol_destroy()
{
	if(ol_xml != NULL)
	{
		gtk_widget_destroy(glade_xml_get_widget(ol_xml, "add_location"));
		g_object_unref(ol_xml);
	}
	ol_xml = NULL;
}
void ol_file_close()
{
}

void ol_file_read(GnomeVFSAsyncHandle *hand, GnomeVFSResult result, gchar *buffer)
{
	if(result == GNOME_VFS_OK)
	{
		gchar **list = g_strsplit(buffer, "\n",0);
		if(list != NULL && list[0] != NULL)
		{
			if(!strncmp(list[0], "[playlist]", 10))
			{
				int i =1;
				while(list[i] != NULL)
				{
					if(!strncmp(list[i], "File", 4))
					{
						mpd_playlist_add(connection, &list[i][6]);
					}

					i++;
				}

			}
			else if(!strncmp(list[0], "http://", 7)){
				int i =0;
				while(list[i] != NULL)
				{
					if(!strncasecmp(list[i], "http://", 7))
					{
						mpd_playlist_add(connection, list[i]);
					}

					i++;
				}
			}
			else{
				GtkEntry *entry = GTK_ENTRY(glade_xml_get_widget(ol_xml, "entry_stream"));
				mpd_playlist_add(connection, (char *)gtk_entry_get_text(entry));
			}
			ol_destroy();
		}
		g_strfreev(list);
	}
	else
	{
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(ol_xml, "label_message")),
				"<span size=\"x-small\"><i>Failed to read the file.</i></span>");
		gtk_widget_set_sensitive(glade_xml_get_widget(ol_xml, "add_location"),TRUE);
	}


	g_free(buffer);


}

void ol_file_opened(GnomeVFSAsyncHandle *hand, GnomeVFSResult result, gpointer data)
{
	gint size = GPOINTER_TO_INT(data);
	size = 1024;
	if(result == GNOME_VFS_OK)
	{
		gchar *buffer = g_malloc0(size+sizeof(char));
		gnome_vfs_async_read(hand, buffer, size,(GnomeVFSAsyncReadCallback)ol_file_read, NULL); 
	}
	else
	{
		g_print("Error found:%s\n", gnome_vfs_result_to_string(result));
		gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(ol_xml, "label_message")),
				"<span size=\"x-small\"><i>Failed to open file.</i></span>");
		gtk_widget_set_sensitive(glade_xml_get_widget(ol_xml, "add_location"),TRUE);
	}
}


void ol_add_location()
{
	int size = 1024;
	GnomeVFSURI *uri;
	if(!gnome_vfs_initialized())
	{
		gnome_vfs_init();
	}
	if(strlen(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(ol_xml, "entry_stream")))) == 0 ) return;

	uri = gnome_vfs_uri_new(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(ol_xml, "entry_stream"))));



	gchar *buffer = g_malloc0(size+sizeof(char));
	gnome_vfs_async_open_uri(&handle, uri,GNOME_VFS_OPEN_READ,GNOME_VFS_PRIORITY_DEFAULT,
			(GnomeVFSAsyncOpenCallback) ol_file_opened, GINT_TO_POINTER(size));

	g_free(buffer);
}


void ol_replace_location()
{
	mpd_playlist_clear(connection);
	ol_add_location();
}



void ol_create_url(GtkWidget *wid,char *url)
{
	gchar *string = NULL;
	/* check if allready open */
	if(ol_xml != NULL)
	{
		gtk_widget_show_all(glade_xml_get_widget(ol_xml, "add_location"));
		gtk_window_present(GTK_WINDOW(glade_xml_get_widget(ol_xml, "add_location")));
		return;
	}

	/* create glade file, and set parent */
	string = g_strdup_printf("%s/osb/osb.glade", plugin.path);
	ol_xml = glade_xml_new(string, "add_location",NULL);
	g_free(string);
	gtk_window_set_transient_for(GTK_WINDOW(glade_xml_get_widget(ol_xml, "add_location")), GTK_WINDOW(wid));


	/* set image with custom stock */
	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(ol_xml, "image")),
			"media-stream", GTK_ICON_SIZE_DIALOG);
	if(url != NULL)
	{
		gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(ol_xml, "entry_stream")),url);
	}

	/* set correct signals */
	glade_xml_signal_connect(ol_xml, "ol_destroy", G_CALLBACK(ol_destroy));
	glade_xml_signal_connect(ol_xml, "ol_add_location", G_CALLBACK(ol_add_location));
}

void ol_create(GtkWidget *wid)
{
	ol_create_url(wid, NULL);
}
